from ..helper import get_singular_name
from ..write_text import write_text
from .examine_action import parse_examine_action


def parse_furniture(json, origin):
    name = get_singular_name(json.get("name", json["id"]))
    if "name" in json:
        write_text(json["name"], origin, comment="Furniture name")
    if "description" in json:
        write_text(json["description"], origin,
                   comment="Description of furniture \"{}\"".format(name))
    if "bash" in json:
        if "sound" in json["bash"]:
            write_text(json["bash"]["sound"], origin,
                       comment="Bashing sound")
        if "sound_fail" in json["bash"]:
            write_text(json["bash"]["sound_fail"], origin,
                       comment="Bashing failed sound")
    if "examine_action" in json:
        parse_examine_action(json["examine_action"], origin,
                             "furniture \"{}\"".format(name))
    if "prying" in json:
        if "message" in json["prying"]:
            write_text(json["prying"]["message"], origin,
                       comment="Prying action message of furniture \"{}\""
                       .format(name))
        if "prying_data" in json["prying"]:
            prying_data = json["prying"]["prying_data"]
            if "failure" in prying_data:
                write_text(prying_data["failure"], origin,
                           comment="Prying failure message of furniture \"{}\""
                           .format(name))
    if "lockpick_message" in json:
        write_text(json["lockpick_message"], origin,
                   comment="Lockpick message of furniture \"{}\"".format(name))



#!/usr/bin/env python3

# Make sure you have python3 installed.
# Ensure that the json_formatter is kept in Tools with this script.
# For Windows:
# Using command prompt type "python name.py"
# For Max OS X or Linux:
# Swap any "\\" with "/", then run the script as in windows.

import json
import os


def gen_new(path):
    change = False
    with open(path, "r", encoding="utf-8") as json_file:
        try:
            json_data = json.load(json_file)
        except UnicodeDecodeError:
            print(f"UnicodeDecodeError in {path}")
            return None
        except json.decoder.JSONDecodeError:
            print(f"JSONDecodeError in {path}")
            return None
        for jo in json_data:
            # We only want JsonObjects
            if type(jo) is str:
                continue
            if not jo.get('name'):
                continue
            if type(jo['name']) is dict:
                continue
            if jo.get('type') not in ['AMMO', 'ARMOR', 'BATTERY', 'bionic',
                                      'BIONIC_ITEM', 'BIONIC_ITEM', 'BOOK',
                                      'COMESTIBLE', 'ENGINE', 'fault',
                                      'GENERIC', 'GUN', 'GUNMOD',
                                      'item_action', 'ITEM_CATEGORY',
                                      'MAGAZINE', 'map_extra', 'martial_art',
                                      'mission_definition', 'MONSTER',
                                      'mutation', 'npc_class', 'PET_ARMOR',
                                      'proficiency', 'skill', 'TOOL',
                                      'TOOL_ARMOR', 'tool_quality', 'TOOLMOD',
                                      'vehicle_part', 'vehicle_part_category',
                                      'vitamin', 'WHEEL', 'SPELL']:
                continue
            if jo.get('name_plural'):
                if jo['name'] == jo['name_plural']:
                    name_obj = {'str_sp': jo['name']}
                else:
                    name_obj = {'str': jo['name'], 'str_pl': jo['name_plural']}
                jo['name'] = name_obj
                del jo['name_plural']
                change = True
            else:
                name_obj = {'str': jo['name']}
                jo['name'] = name_obj
                change = True

    return json_data if change else None


def change_file(json_dir):
    for root, directories, filenames in os.walk(f"..\\{json_dir}"):
        for filename in filenames:
            path = os.path.join(root, filename)
            if path.endswith(".json"):
                new = gen_new(path)
                if new is not None:
                    with open(path, "w", encoding="utf-8") as jf:
                        json.dump(new, jf, ensure_ascii=False)
                    os.system(f".\\json_formatter.exe {path}")


if __name__ == "__main__":
    json_dir = input("What directory are the json files in? ")
    change_file(json_dir)



#!/usr/bin/env python3
"""
Generate sprites for mapgen IDs
"""
import argparse
import json

from collections import OrderedDict
from collections import defaultdict
from pathlib import Path
from typing import Optional, Union

from PIL import Image

from util import import_data


SIZE = 24
ROTATIONS = ('_east', '_south', '_west')

TILE_ENTRY_TEMPLATE = {
    'id': '',
    'fg': '',
    'rotates': True,
}
SKIPPED = {
    'duplicate': set(),
    'predecessor_mapgen': list(),
    'no_rows': list(),
}
TERRAIN_COLOR_NAMES = {}
PALETTES = {}
OVERMAP_TERRAIN_DATA = {}
SCHEME = {}
CREATED_IDS = set()
VARIANTS = dict()
SINGLE_COLOR_SPRITES = defaultdict(set)
COLOR_NAMES = set()


def get_first_valid(
    value: Union[str, list, OrderedDict],
    default: Optional[str] = None,
) -> str:
    """
    Return first str value from a list [of lists] that is not empty

    TODO: support all quirks properly and get rid of this function
    """
    if isinstance(value, OrderedDict):
        return value.get('fallback')

    while isinstance(value, list):
        for item in value:
            if item:
                value = item
                break
    if value:
        return value
    return default


class TupleJSONDecoder(json.JSONDecoder):
    """
    Decode JSON lists as tuples
    https://stackoverflow.com/a/10889125
    """

    def __init__(self, **kwargs):
        json.JSONDecoder.__init__(self, **kwargs)
        # Use the custom JSONArray
        self.parse_array = self.JSONArray
        # Use the python implemenation of the scanner
        self.scan_once = json.scanner.py_make_scanner(self)

    def JSONArray(self, s_and_end, scan_once, **kwargs):
        values, end = json.decoder.JSONArray(s_and_end, scan_once, **kwargs)
        return tuple(values), end


def generate_image(
    rows: list,
    terrain_dict: dict,
    fill_ter: str,
    # rotation: int = 0,
) -> Image:
    """
    Generate sprite from rows
    """
    image = Image.new('RGB', [len(rows[0]), len(rows)])
    image_data = image.load()
    for index_x, row in enumerate(rows):
        for index_y, char in enumerate(row):
            terrain_id = get_first_valid(terrain_dict.get(char), fill_ter)
            # TODO: skip on t_secretdoor_metal_c ?

            color = SCHEME['override_terrain_color'].get(terrain_id)
            if not color:
                color = SCHEME['colors'].get(
                    TERRAIN_COLOR_NAMES.get(terrain_id))
            if not color:
                color = SCHEME['override_terrain_color'].get(
                    't_null',
                    (0, 0, 0)
                )

            try:
                image_data[index_y, index_x] = color
            except TypeError:
                print(color)
                raise

    # if rotation:
    #    image = image.rotate(-90 * rotation, expand=True)

    return image


def split_image(
    image: Image,
    om_ids: list,
) -> tuple:
    """
    Split image into SIZExSIZE sprites, yield (id, image) pairs
    """
    width, height = image.size
    for row in range(height // SIZE):
        for col in range(width // SIZE):
            box = (col * SIZE, row * SIZE, (col + 1) * SIZE, (row + 1) * SIZE)
            output = image.crop(box)
            yield om_ids[row][col], output


def read_scheme(
    color_scheme_path: Path,
) -> None:
    """
    Parse color scheme JSON into the SCHEME global
    """
    with open(color_scheme_path) as file:
        data = json.load(file, cls=TupleJSONDecoder)

    SCHEME['colors'] = data['colors']
    SCHEME['replace_with_context'] = data.get('replace_with_context', [])

    SCHEME['override_terrain_color'] = {}
    for color, terrain_ids in data.get('override_terrain_color'):
        if isinstance(terrain_ids, str):
            terrain_ids = (terrain_ids,)
        for terrain_id in terrain_ids:
            SCHEME['override_terrain_color'][terrain_id] = color

    # TODO: support fallback_predecessor_mapgen
    SCHEME['override_terrain_color'][None] = 0, 0, 0, 0


def read_terrain_color_names() -> None:
    """
    Fill the TERRAIN_COLOR_NAMES global
    """
    terrain_data, errors = import_data(
        json_dir=Path('../../data/json/furniture_and_terrain/'),
        json_fmatch='terrain*.json',
    )
    if errors:
        print(errors)

    for terrain in terrain_data:
        terrain_type = terrain.get('type')
        terrain_id = terrain.get('id')
        terrain_color = terrain.get('color')
        if isinstance(terrain_color, list):
            terrain_color = terrain_color[0]
        if terrain_type == 'terrain' and terrain_id and terrain_color:
            TERRAIN_COLOR_NAMES[terrain_id] = terrain_color
            COLOR_NAMES.add(terrain_color)


def read_overmap_terrain_data() -> None:
    """
    Fill the OVERMAP_TERRAIN_COLORS
    """
    overmap_terrain_data, errors = import_data(
        json_dir=Path('../../data/json/overmap/overmap_terrain/'),
        json_fmatch='*.json',
    )
    if errors:
        print(errors)

    for entry in overmap_terrain_data:
        if entry.get('type') != 'overmap_terrain':
            continue
        entry_ids = entry.get('id')
        if not entry_ids:
            continue

        color_name = entry.get('color')
        COLOR_NAMES.add(color_name)

        color = SCHEME['colors'].get(color_name)
        if not color:
            continue

        if isinstance(entry_ids, str):
            entry_ids = (entry_ids,)

        for terrain_id in entry_ids:
            OVERMAP_TERRAIN_DATA[terrain_id] = color


def add_palette(
    entry: dict,
) -> None:
    """
    Add new palette to the PALETTES global
    """
    if entry.get('type') != 'palette':
        return

    terrains = entry.get('terrain')
    if terrains is None:
        terrains = {}

    palette_id = entry.get('id')

    if palette_id in PALETTES:
        raise Exception(f'WARNING: duplicate palette id {palette_id}')

    PALETTES[palette_id] = {
        key: value for
        key, value in
        terrains.items() if
        not isinstance(value, dict) and
        not isinstance(value, OrderedDict)
        # TODO: support whatever this is
    }


def read_mapgen_palettes() -> None:
    """
    Fill the PALETTES global
    """
    palette_entries, errors = import_data(
        json_dir=Path('../../data/json/mapgen_palettes/'),
        json_fmatch='*.json',
    )
    if errors:
        print(errors)

    for entry in palette_entries:
        add_palette(entry)


def get_mapgen_data(
    mapgen_dir: Path,
    pattern: str,
) -> list:
    """
    Get all mapgen entries
    """
    mapgen_data, errors = import_data(
        json_dir=mapgen_dir,
        json_fmatch=pattern,
    )
    if errors:
        print(errors)

    return mapgen_data


def write_image_with_rotations(
    image: Image,
    name: str,
    output_dir: Path,
    # raw_rotation: Union[None, int, list] = None,
) -> None:
    """
    Write image to disk along with the rotations
    """
    context_color = OVERMAP_TERRAIN_DATA.get(name)
    if context_color:
        pixels = image.load()
        height, width = image.size
        for col in range(height):
            for row in range(width):
                if pixels[col, row] in SCHEME['replace_with_context']:
                    pixels[col, row] = context_color

    image.save(output_dir / f'{name}.png')
    for rotation_suffix in ROTATIONS:
        image = image.rotate(-90)
        image.save(output_dir / f'{name}{rotation_suffix}.png')


def write_json(
    output_dir: Path,
    name: str,
) -> None:
    """
    Generate and write JSON tile entry for compose.py
    """
    tile_entry = TILE_ENTRY_TEMPLATE.copy()
    tile_entry['id'] = name
    tile_entry['fg'] = [name] + [f'{name}{suffix}' for suffix in ROTATIONS]
    filepath = output_dir / f'{name}.json'
    with open(filepath, 'w') as file:
        json.dump(tile_entry, file)


def output_sprite(
    name: str,
    image: Image,
    generate_json: bool,
    output_dir: Optional[Path] = None,
    duplicates_dir: Optional[Path] = None,
    rotation: int = 0,
    # rotation: Union[None, int, list] = None,
    # single_terrain: bool = False
) -> None:
    """
    Handle single sprite
    """
    for suffix in ROTATIONS:
        if f'{name}{suffix}' in CREATED_IDS:
            raise Exception(
                'Rotation suffix caused duplicated sprite names: '
                f'{name}{suffix}'
            )

    if rotation:
        image = image.rotate(-90 * rotation)  # , expand=True

    if name not in CREATED_IDS:
        # new ID

        image_colors = image.getcolors()
        if len(image_colors) < 2:  # and not single_terrain
            SINGLE_COLOR_SPRITES[image_colors[0][1]].add(name)
            return

        if output_dir is None:
            CREATED_IDS.add(name)
            return

        if generate_json:
            write_json(output_dir=output_dir, name=name)

        write_image_with_rotations(image, name, output_dir)  # , rotation)
        CREATED_IDS.add(name)
        return

    if not duplicates_dir:
        # skipping as duplicate
        SKIPPED['duplicate'].add(name)
        return

    duplicates_subdir = duplicates_dir / name

    if name not in VARIANTS:
        # newly discovered ID with variants, move the previously generated one
        # into a duplicates subdirectory
        duplicates_subdir.mkdir(parents=True, exist_ok=True)
        (output_dir / f'{name}.png').rename(
            duplicates_subdir / f'{name}.png'
        )
        for rotation in ROTATIONS:
            (output_dir / f'{name}{rotation}.png').rename(
                duplicates_subdir / f'{name}{rotation}.png'
            )
        if generate_json:
            (output_dir / f'{name}.json').rename(
                duplicates_subdir / f'{name}.json'
            )
        VARIANTS[name] = 2

    else:
        # ID is known to have variants, just increment the counter
        VARIANTS[name] += 1

    write_image_with_rotations(
        image,
        f'{name}_{VARIANTS[name]}',
        duplicates_subdir,
        # rotation,
    )


def get_initial_rotation(rotation: Union[None, int, list]) -> int:
    """
    >>> get_initial_rotation(None)
    0
    >>> get_initial_rotation(1)
    1
    >>> get_initial_rotation([2])
    2
    >>> get_initial_rotation([3, 3])
    3
    >>> get_initial_rotation([0, 3])
    0
    >>> get_initial_rotation([3, 4])
    WARNING: unexpected rotation value [3, 4]
    0
    """
    if not rotation:
        return 0

    if isinstance(rotation, int):
        return rotation

    if not isinstance(rotation, list):
        raise Exception(f'Unexpected rotation value: {rotation}')

    if len(rotation) == 1:
        return rotation[0]

    if len(rotation) != 2:
        raise Exception(f'Unexpected rotations length: {rotation}')

    if rotation[0] == rotation[1]:
        return rotation[0]

    if rotation in ([0, 3], [0, 1]):
        return 0

    # if min(rotation) < 0 or max(rotation) > 3:
    print(f'WARNING: unexpected rotation value {rotation}')

    return 0


def handle_single_color_sprites(
    output_dir: Path,
) -> None:
    """
    Generate images for all colors and create JSON with all the tile entries
    """
    entries = list()

    output_dir /= 'single_color'
    output_dir.mkdir(parents=True, exist_ok=True)

    for color, names in SINGLE_COLOR_SPRITES.items():
        image = Image.new('RGB', [SIZE, SIZE], color)
        color_str = '_'.join(map(str, color))
        image.save(output_dir / f'{color_str}.png')

        entry = {
            'id': list(names),
            'fg': color_str,
        }
        entries.append(entry)

    with open(output_dir / 'entries.json', 'w') as file:
        json.dump(entries, file)


def main():
    """
    main
    """
    # get arguments
    arg_parser = argparse.ArgumentParser(
        description=__doc__,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    arg_parser.add_argument(
        'color_scheme', type=Path,  # TODO: default
        help='path to a JSON color SCHEME',
    )
    arg_parser.add_argument(
        'output_dir', nargs='?', type=Path,
        help='Output directory path',
    )
    arg_parser.add_argument(
        '--json', dest='json', action='store_true',
        help='Generate JSON files for correct rotations',
    )
    arg_parser.add_argument(
        '--context-coloring', dest='context_coloring', action='store_true',
        help='Replace colors specified in the color scheme with '
        'overmap_terrain color',
    )
    arg_parser.add_argument(
        '--mapgen-dir', dest='mapgen_dir', type=Path,
        default=Path('../../data/json/mapgen/'),
        help='directory with mapgen entries',
    )
    arg_parser.add_argument(
        '--mapgen-files-pattern', dest='mapgen_files_pattern', type=str,
        default='*.json',
        help='filename template for selecting a subset '
        'of JSON files in the mapgen dir',
    )
    arg_parser.add_argument(
        '--duplicates-dir', dest='duplicates_dir', type=Path,
        help='directory for putting duplicate ID sprites into. '
        'Should not be in the output_dir when using compose.py --use-all',
    )
    args_dict = vars(arg_parser.parse_args())

    output_dir = args_dict.get('output_dir', None)
    duplicates_dir = args_dict.get('duplicates_dir', None)
    color_scheme_path = args_dict.get('color_scheme')
    generate_json = args_dict.get('json')
    context_coloring = args_dict.get('context_coloring')

    # read needed values from the data/json/ subdirectories
    read_scheme(color_scheme_path)

    read_terrain_color_names()

    read_mapgen_palettes()

    if context_coloring:
        if SCHEME['replace_with_context']:
            read_overmap_terrain_data()
        else:
            print(
                'WARNING: replace_with_context color scheme value is empty '
                'but context_coloring is True'
            )

    mapgen_data = get_mapgen_data(
        mapgen_dir=args_dict.get('mapgen_dir'),
        pattern=args_dict.get('mapgen_files_pattern'),
    )

    # get palettes
    for entry in mapgen_data:
        if entry.get('type') == 'palette':
            add_palette(entry)

    # main loop over mapgens
    for entry in mapgen_data:
        # check type
        if entry.get('type') != 'mapgen':
            continue

        # check method, only json supported currently
        if entry.get('method') != 'json':
            continue

        om_id = entry.get('om_terrain')
        if not om_id or om_id == 'FEMA_entrance':
            continue

        # verify that "object" value is defined
        mapgen = entry.get('object')
        if not mapgen:
            continue

        # combine "palettes" value with "terrain" to get all terrain values
        terrain_dict = {}
        palettes = mapgen.get('palettes')
        if palettes:
            for palette_id in palettes:
                if isinstance(palette_id, OrderedDict):
                    palette_id = get_first_valid(
                        palette_id.get('distribution')
                    )
                palette = PALETTES.get(palette_id)
                if palette is not None:
                    terrain_dict.update(palette)
                else:
                    print(f'WARNING: unknown palette {palette_id}')
                # TODO: support nested palettes, two cases found

        terrain_defs = mapgen.get('terrain')
        if terrain_defs:
            terrain_dict.update(terrain_defs)

        # single_terrain = False
        # if not terrain_dict:
        #    single_terrain = True

        raw_rotation = mapgen.get('rotation')

        # verify "rows" is not empty
        rows = mapgen.get('rows')
        fill_ter = mapgen.get('fill_ter')

        if not rows:
            if fill_ter:
                rows = [' ' * SIZE] * SIZE
            else:
                if mapgen.get('predecessor_mapgen'):
                    SKIPPED['predecessor_mapgen'].append(om_id)
                else:
                    SKIPPED['no_rows'].append(om_id)
                continue

        # create the sprite
        image = generate_image(
            rows=rows, terrain_dict=terrain_dict, fill_ter=fill_ter,
        )

        # write sprite[s] to the output directory
        if isinstance(om_id, list) and isinstance(om_id[0], list):
            generator = split_image(image=image, om_ids=om_id)
            for submap_id, submap_image in generator:
                output_sprite(
                    name=submap_id,
                    image=submap_image,
                    generate_json=generate_json,
                    output_dir=output_dir,
                    duplicates_dir=duplicates_dir,
                    rotation=get_initial_rotation(raw_rotation),
                    # single_terrain=single_terrain,
                )

        else:
            om_id = get_first_valid(om_id)
            output_sprite(
                name=om_id,
                image=image,
                generate_json=generate_json,
                output_dir=output_dir,
                duplicates_dir=duplicates_dir,
                rotation=get_initial_rotation(raw_rotation),
                # single_terrain=single_terrain,
            )

    if output_dir:
        handle_single_color_sprites(output_dir)


if __name__ == '__main__':
    main()

    print(f'generated {len(CREATED_IDS)} sprites')
    print('Skipped:')
    for reason, skipped_values in SKIPPED.items():
        print(reason, len(skipped_values))
        print(skipped_values or '')
    print('Unknown color names:')
    print(COLOR_NAMES - SCHEME['colors'].keys())



from ..helper import get_singular_name
from ..write_text import write_text


def parse_npc_class(json, origin):
    comment = json.get("//", None)
    name = ""
    if "name" in json:
        name = get_singular_name(json["name"])
        write_text(json["name"], origin,
                   comment=["Name of an NPC class", comment])
    elif "id" in json:
        name = json["id"]
    if "job_description" in json:
        write_text(json["job_description"], origin,
                   comment=["Job description of \"{}\" NPC class".format(name),
                            comment])
    if "shopkeeper_item_group" in json:
        for group in json["shopkeeper_item_group"]:
            if "refusal" in group:
                commentg = group.get("//", None)
                write_text(group["refusal"], origin,
                           comment=["Refusal reason for a shop item group",
                                    commentg])



from ..write_text import write_text


def parse_profession(json, origin):
    name_male = ""
    name_female = ""
    if "name" not in json:
        name_male = name_female = json["id"]
    elif type(json["name"]) is dict:
        name_male = json["name"]["male"]
        name_female = json["name"]["female"]
    elif type(json["name"]) is str:
        name_male = name_female = json["name"]

    desc_male = ""
    desc_female = ""
    has_description = "description" in json
    if has_description:
        if type(json["description"]) is dict:
            if "male" in json["description"]:
                desc_male = json["description"]["male"]
                desc_female = json["description"]["female"]
            elif "str" in json["description"]:
                desc_male = desc_female = json["description"]["str"]
        else:
            desc_male = desc_female = json["description"]

    write_text(name_male, origin, context="profession_male",
               comment="Profession name for male")
    if has_description:
        write_text(desc_male, origin, context="prof_desc_male",
                   comment="Description of profession \"{}\" for male".
                   format(name_male))

    write_text(name_female, origin, context="profession_female",
               comment="Profession name for female")
    if has_description:
        write_text(desc_female, origin, context="prof_desc_female",
                   comment="Description of profession \"{}\" for female".
                   format(name_female))



#!/usr/bin/env python3

"""

Example usage:
py monstergroup_name_to_id.py ../../data/mods/my_mod

See https://github.com/CleverRaven/Cataclysm-DDA/pull/79177
Should be removed after 0.I

"""
import argparse
import json
import os

args = argparse.ArgumentParser()
args.add_argument("dir", action="store", help="specify json directory")
args_dict = vars(args.parse_args())

failures = set()


def format_json(path):
    file_path = os.path.dirname(__file__)
    # Might need changing to match your setup
    format_path_linux = os.path.join(file_path, "../format/json_formatter.cgi")
    path_win = "../../json_formatter.exe"
    alt_path_win = "../format/json_formatter.exe"
    format_path_win = os.path.join(file_path, path_win)
    format_alt_path_win = os.path.join(file_path, alt_path_win)
    if os.path.exists(format_path_linux):
        os.system(f"{format_path_linux} {path}")
    elif os.path.exists(format_path_win):
        os.system(f"{format_path_win} {path}")
    elif os.path.exists(format_alt_path_win):
        os.system(f"{format_alt_path_win} {path}")
    else:
        print("No json formatter found")


def readWriteJson(path, jo_function):
    change = False
    format_error = "Json Decode Error at:\n{0}\nEnsure that the file is a "\
        "JSON file consisting of an object or array of objects!".format(path)
    with open(path, "r", encoding="utf-8") as json_file:
        try:
            json_data = json.load(json_file)
        except json.JSONDecodeError:
            failures.add(format_error)
            return
        if type(json_data) is dict:  # File with a single json object
            json_data = jo_function(json_data)
            change = json_data is not None
        elif type(json_data) is list:
            for jo in json_data:
                if type(jo) is not dict:
                    failures.add(format_error)
                    return
                jo_new = jo_function(jo)
                changed = jo_new is not None
                if changed:
                    change = True
                    pos = json_data.index(jo)
                    json_data.insert(pos, jo_new)
                    json_data.remove(jo)
        else:
            failures.add(format_error)
        if change:
            with open(path, "w", encoding="utf-8") as jf:
                json.dump(json_data, jf, ensure_ascii=False)
            format_json(path)


def migrateMonsterGroupNameToId(jo):
    assert "type" in jo
    if jo["type"] == "monstergroup" and "name" in jo:
        assert type(jo["name"]) is str
        # While dicts are ordered they have no way to directly edit keys
        # or access by index
        pos = list(jo.keys()).index("name")
        items = list(jo.items())
        items.insert(pos, ("id", jo["name"]))
        new_jo = dict(items)
        new_jo.pop("name")
        return new_jo
    return None


for root, directories, filenames in os.walk(args_dict["dir"]):
    for filename in filenames:
        path = os.path.join(root, filename)
        if path.endswith(".json"):
            readWriteJson(path, migrateMonsterGroupNameToId)

for statement in failures:
    print(statement)



from .message import Message, messages, occurrences


def append_comment(comments, new_comment):
    if type(new_comment) is str:
        return comments + new_comment.split("\n")
    elif type(new_comment) is list:
        for comment in new_comment:
            if comment:
                comments = append_comment(comments, comment)
        return comments


def write_text(json, origin, context="", comment="",
               plural=False, c_format=True):
    """
    Record a text for translation.

    Parameters:
        json: The text in string or JSON dict form
        origin (str): Path of JSON source location
        context (str): "context" as in GNU gettext
        comment: Translation comments in either string form or list of strings
        plural (bool): Whether the text should be pluralized
        c_format (bool): Whether the text contains C-style format string
        explicit_plural (bool): Whether the plural is specified
                                explicitly in JSON
    """
    if json is None or json == "":
        return

    comments = append_comment([], comment)
    text = ""
    text_plural = ""
    explicit_plural = False

    if type(json) is str:
        text = json
        if plural:
            text_plural = "{}s".format(text)
    elif type(json) is dict:
        if "//~" in json:
            if type(json["//~"]) is str and json["//~"]:
                comments = append_comment(comments, json["//~"])
        if "ctxt" in json:
            if type(json["ctxt"]) is str:
                context = json["ctxt"]
        if "str" in json:
            text = json["str"]
        if plural:
            if "str_sp" in json:
                text = json["str_sp"]
                text_plural = json["str_sp"]
                explicit_plural = True
            elif "str_pl" in json:
                text_plural = json["str_pl"]
                explicit_plural = True
            else:
                text_plural = "{}s".format(text)

    if not text or "NO_I18N" in comments:
        return

    format_tag = ""
    if "%" in text:
        if c_format:
            format_tag = "c-format"
        else:
            format_tag = "no-c-format"

    if (context, text) not in messages:
        messages[(context, text)] = list()

    messages[(context, text)].append(
        Message(comments, origin, format_tag, context,
                text, text_plural, explicit_plural))
    occurrences.append((context, text))


# Used in parse_effect and parse_condition
def write_translation_or_var(json, origin, context="", comment="",
                             plural=False, c_format=True):
    if type(json) is dict and "default_str" in json:
        write_text(json["default_str"], origin, context=context,
                   comment="default value for {}".format(comment),
                   plural=plural, c_format=c_format)
    else:
        write_text(json, origin, context=context, comment=comment,
                   plural=plural, c_format=c_format)



from ..write_text import write_text


def parse_json_flag(json, origin):
    if "info" in json:
        write_text(json["info"], origin, c_format=False, comment=[
            "Please leave anything in <angle brackets> unchanged.",
            "Description of JSON flag \"{}\"".format(json["id"])
        ])
    if "restriction" in json:
        write_text(json["restriction"], origin, c_format=False,
                   comment="Description of restriction of JSON flag \"{}\""
                   .format(json["id"]))
    if "item_prefix" in json:
        write_text(json["item_prefix"], origin, c_format=False,
                   comment="This is custom prefix added to item name. \"{}\""
                   .format(json["id"]))
    if "item_suffix" in json:
        write_text(json["item_suffix"], origin, c_format=False,
                   comment="This is custom suffix added to item name. \"{}\""
                   .format(json["id"]))



import itertools

from .effect import parse_effect
from ..helper import is_tag
from ..write_text import write_text


all_genders = ["f", "m", "n"]


def gender_options(subject):
    return [subject + ":" + g for g in all_genders]


dynamic_line_string_keys = [
    # from `parsers_simple` in `condition.cpp`
    "u_male", "u_female", "npc_male", "npc_female",
    "has_no_assigned_mission", "has_assigned_mission",
    "has_many_assigned_missions", "has_no_available_mission",
    "has_available_mission", "has_many_available_missions",
    "mission_complete", "mission_incomplete", "mission_has_generic_rewards",
    "npc_available", "npc_following", "npc_friend", "npc_hostile",
    "npc_train_skills", "npc_train_styles", "npc_train_spells",
    "at_safe_space", "is_day", "npc_has_activity",
    "u_is_travelling", "npc_is_travelling",
    "is_outside", "u_is_outside", "npc_is_outside", "u_has_camp",
    "u_can_stow_weapon", "npc_can_stow_weapon", "u_can_drop_weapon",
    "npc_can_drop_weapon", "u_has_weapon", "npc_has_weapon",
    "u_driving", "npc_driving", "has_pickup_list", "is_by_radio", "has_reason",
    "u_is_avatar", "npc_is_avatar", "u_is_npc", "npc_is_npc",
    "u_is_character", "npc_is_character", "u_is_monster", "npc_is_monster",
    "u_is_item", "npc_is_item", "u_is_furniture", "npc_is_furniture",
    "player_see_u", "player_see_npc",
    "u_is_driven", "npc_is_driven",
    "u_is_remote_controlled", "npc_is_remote_controlled",
    "u_can_fly", "npc_can_fly",
    "u_is_flying", "npc_is_flying",
    "u_is_in_air", "npc_is_in_air",
    "u_can_float", "npc_can_float",
    "u_is_floating", "npc_is_floating",
    "u_is_falling", "npc_is_falling",
    "u_is_skidding", "npc_is_skidding",
    "u_is_sinking", "npc_is_sinking",
    "u_is_on_rails", "npc_is_on_rails",
    "u_is_avatar_passenger", "npc_is_avatar_passenger",
    # yes/no strings for complex conditions, 'and' list
    "yes", "no", "concatenate"
]


def parse_dynamic_line(json, origin, comment=[]):
    if type(json) is list:
        for line in json:
            parse_dynamic_line(line, origin, comment)

    elif type(json) is dict:
        if "str" in json:
            write_text(json, origin, comment=comment, c_format=False)

        if "gendered_line" in json:
            text = json["gendered_line"]
            subjects = json["relevant_genders"]
            options = [gender_options(subject) for subject in subjects]
            gendered_comment = [
                *comment,
                "You don't need to translate genders that "
                "aren't in your language",
                "Check: https://github.com/CleverRaven/"
                "Cataclysm-DDA/blob/master/doc/"
                "TRANSLATING.md#grammatical-gender"]
            for context_list in itertools.product(*options):
                context = " ".join(context_list)
                write_text(text, origin, context=context,
                           comment=gendered_comment, c_format=False)

        for key in dynamic_line_string_keys:
            if key in json:
                parse_dynamic_line(json[key], origin, comment=comment)

    elif type(json) is str:
        if not is_tag(json):
            write_text(json, origin, comment=comment, c_format=False)


def parse_response(json, origin):
    if "text" in json:
        write_text(json["text"], origin, c_format=False,
                   comment="Response to NPC dialogue")

    if "truefalsetext" in json:
        write_text(json["truefalsetext"]["true"], origin, c_format=False,
                   comment="Affirmative response to NPC dialogue")
        write_text(json["truefalsetext"]["false"], origin, c_format=False,
                   comment="Negative response to NPC dialogue")

    if "failure_explanation" in json:
        write_text(json["failure_explanation"], origin, c_format=False,
                   comment="Failure explanation for NPC dialogue response")

    if "success" in json:
        parse_response(json["success"], origin)

    if "failure" in json:
        parse_response(json["failure"], origin)

    if "speaker_effect" in json:
        speaker_effects = json["speaker_effect"]
        if not type(speaker_effects) is list:
            speaker_effects = [speaker_effects]
        for eff in speaker_effects:
            if "effect" in eff:
                parse_effect(eff["effect"], origin)

    if "effect" in json:
        parse_effect(json["effect"], origin)


def parse_talk_topic(json, origin):
    if "dynamic_line" in json:
        comment = ["NPC dialogue line"]
        parse_dynamic_line(json["dynamic_line"], origin, comment=comment)

    if "responses" in json:
        for response in json["responses"]:
            parse_response(response, origin)

    if "repeat_responses" in json:
        if type(json["repeat_responses"]) is dict:
            if "response" in json["repeat_responses"]:
                parse_response(json["repeat_responses"]["response"], origin)
        elif type(json["repeat_responses"]) is list:
            for response in json["repeat_responses"]:
                if "response" in response:
                    parse_response(response["response"], origin)

    if "effect" in json:
        parse_effect(json["effect"], origin)



#!/usr/bin/env python3
from spell_checker.spell_checker import spell_check
import polib


pofile = polib.pofile('./lang/po/cataclysm-dda.pot')
# occurrences = dict()
for entry in pofile:
    typos = spell_check(entry.msgid)
    # for word in typos:
    #     if word in occurrences:
    #         occurrences[word] += 1
    #     else:
    #         occurrences[word] = 1
    if typos:
        print(typos, "<=", entry.msgid.replace('\n', '\\n'))
    if entry.msgid_plural:
        typos = spell_check(entry.msgid_plural)
        if typos:
            print(typos, "<=", entry.msgid_plural.replace('\n', '\\n'))



from ..write_text import write_text


def parse_widget(json, origin):
    id = json["id"]
    if "label" in json:
        write_text(json["label"], origin,
                   comment="Label of UI widget \"{}\"".format(id))
    if "description" in json:
        write_text(json["description"], origin,
                   comment="Description of UI widget \"{}\"".format(id))
    if "string" in json:
        write_text(json["string"], origin,
                   comment="Text in UI widget \"{}\"".format(id))
    if "phrases" in json:
        for phrase in json["phrases"]:
            comment = "Text in portion of UI widget \"{}\"".format(id)
            if "text" in phrase:
                write_text(phrase["text"], origin, comment=comment)
    if "default_clause" in json and "text" in json["default_clause"]:
        write_text(json["default_clause"]["text"], origin,
                   comment="Default clause of UI widget \"{}\"".format(id))
    if "clauses" in json:
        for clause in json["clauses"]:
            if "text" not in clause:
                continue
            write_text(clause["text"], origin,
                       comment="Clause text \"{}\" of UI widget \"{}\""
                       .format(clause["id"], id))



#!/usr/bin/env python3

import argparse
import glob
import os
import sys
import subprocess


class IncludesParser:
    def __init__(self):
        self.includes_to_sources = {}

    def parse_includes_file(
            self,
            includes_file,
            source_file_path,
            relative_root,
    ):
        includes = includes_file.read().splitlines()
        for include in includes:
            if len(include) == 0 or include[0] != '.':
                # Out of actual output
                # gcc additionally prints an informational section like:
                # Multiple include guards may be useful for:
                break

            # Format is one or more '.' characters, followed by a space,
            # followed by the header
            include_actual = include[include.index(' ') + 1:]
            if include_actual[0] == '/':
                # system header, don't care
                continue

            include_actual = os.path.normpath(
                os.path.join(relative_root, include_actual))
            self.includes_to_sources.setdefault(
                include_actual,
                set(),
            ).add(source_file_path)

        # A source file also 'includes' itself
        self.includes_to_sources.setdefault(
            source_file_path,
            set(),
        ).add(source_file_path)

    # Parses includes files from a given folder
    # Maps include file path to a source file path by converting
    # folder_path/*.inc -> file_path_prefix/*.cpp
    def parse_includes_files_from(
            self,
            folder_path,
            source_path_prefix,
            include_path_prefix,
    ):
        # Load includes files list.
        folder_len = len(folder_path) + 1
        includes_files = glob.glob(
            os.path.join(folder_path, "**/*.inc"),
            recursive=True,
        )
        if len(includes_files) == 0:
            print(
                f"No includes files found in {folder_path}, " +
                "did you run `make includes`?",
                file=sys.stderr,
            )
            return False

        for includes_file_path in includes_files:
            source_file = includes_file_path[folder_len:-3] + "cpp"
            # Generally folder_path is <something>/obj,
            # but results might include `tiles`
            if source_file.startswith("tiles/"):
                source_file = source_file[6:]

            source_file = os.path.join(source_path_prefix, source_file)
            with open(includes_file_path, 'r') as includes_file:
                self.parse_includes_file(
                    includes_file,
                    source_file,
                    include_path_prefix,
                )

        return True

    def get_files_affected_by(self, changed_file):
        return self.includes_to_sources.get(changed_file, [])


def get_changed_files_from_list(changed_files_list):
    with open(changed_files_list, 'r') as f:
        changed_files = [line.strip() for line in f.read().splitlines()]
        return changed_files


def get_changed_files_from_git(merge_target):
    out = subprocess.run(
        ["git", "diff", "--merge-base", "--name-only", merge_target],
        capture_output=True)
    if out.returncode != 0:
        print(out.stderr, file=sys.stderr)
        raise Exception("git diff failed to run")
    changed_files = [line.strip() for line in out.stdout.decode().splitlines()]
    return changed_files


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("--changed-files-list", help=("Path to a file"
                        "containing list of changed files, one per line."))
    parser.add_argument(
        "--merge-target", default="master",
        help=("Name of the branch we are intending to merge into."
              "Will be used to determine the list of files changed. "
              "Ignored if --changed-files-list specified explicilty."))
    args = parser.parse_args()

    changed_files = None
    if args.changed_files_list:
        changed_files = get_changed_files_from_list(args.changed_files_list)
    else:
        changed_files = get_changed_files_from_git(args.merge_target)

    parser = IncludesParser()
    # Load includes files list.
    if not parser.parse_includes_files_from("obj", "src", "."):
        return 1

    # Also for tests
    if not parser.parse_includes_files_from("tests/obj", "tests", "tests"):
        return 1

    lintable_files = set()
    for changed_file in changed_files:
        for affected_file in parser.get_files_affected_by(changed_file):
            lintable_files.add(affected_file)
    changed_files = set(changed_files)

    # Lightly sort the output list so that directly changed files appear first
    lintable_files_sorted = []
    lintable_files_sorted.extend(sorted(lintable_files & changed_files))
    lintable_files_sorted.extend(sorted(lintable_files - changed_files))

    for lintable_file in lintable_files_sorted:
        print(lintable_file)

    return 0


if __name__ == '__main__':
    sys.exit(main())



#!/usr/bin/env python3

import json
import argparse
from math import hypot
import re
import os

args = argparse.ArgumentParser(
    description="Generate a json definition for a vehicle "
    "in a Cataclysm DDA save file.")
args.add_argument(
    "save", action="store", help="specify save file containing vehicle")
args.add_argument(
    "vehicle", nargs="?", help="specify name of vehicle", default=None)
argsDict = vars(args.parse_args())


def writeVehicleTemplates(templates):
    with open("vehicles.json", "w", encoding="utf-8") as vehicleDefJson:
        json.dump(templates, vehicleDefJson, indent=4, ensure_ascii=False)
        print("Vehicle defs written.")


def getVehicleTemplates():
    vehicles = []
    for root, directories, filenames in os.walk(argsDict["save"]):
        for filename in filenames:
            path = os.path.join(root, filename)
            if path.endswith(".map"):
                vehicles += getVehicleInstances(path)

    allTemplates = []
    for vehicle in vehicles:
        vehicleDef = buildVehicleDef(vehicle)
        if vehicleDef not in allTemplates:
            allTemplates.append(vehicleDef)

    return allTemplates


def getVehicleInstances(mapPath):
    vehicles = []
    with open(mapPath, encoding="utf-8") as mapFile:
        mapData = json.load(mapFile)
        for i in range(0, len(mapData)):
            for vehicle in mapData[i]["vehicles"]:
                if argsDict["vehicle"] is not None:
                    if argsDict["vehicle"] == vehicle["name"]:
                        vehicles.append(vehicle)
                        print(f"Found \"{vehicle['name']}\"")
                else:
                    vehicles.append(vehicle)
                    print(f"Found \"{vehicle['name']}\"")

    return vehicles


def buildVehicleDef(vehicle):
    partsDef = []
    itemsDef = []
    for part in vehicle["parts"]:
        part_variant = ""
        if "variant" in part:
            part_variant = "_" + part["variant"]

        partsDef.append({
            "x": part["mount_dx"],
            "y": part["mount_dy"],
            "part": part["id"] + part_variant
        })

        for item in part["items"]:
            itemsDef.append({
                "x": part["mount_dx"],
                "y": part["mount_dy"],
                "chance": 100,
                "items": [item["typeid"]]
            })

    frames = [
        p for p in partsDef
        if re.match(r'(xl|hd|folding_)?frame', p["part"]) is not None
    ]
    everythingElse = [
        p for p in partsDef
        if re.match(r'(xl|hd|folding_)?frame', p["part"]) is None
    ]

    frames = sortFrames(frames)

    itemsDef.sort(key=lambda i: (i["x"], i["y"]))

    vehicleDef = {
        "id": vehicle["name"],
        "type": "vehicle",
        "name": vehicle["name"],
        "parts": frames + everythingElse,
        "items": itemsDef
    }

    return vehicleDef


def sortFrames(frames):
    sortedFrames = []
    sortedFrames.append(frames.pop())

    while len(frames) > 0:
        nextFrame = frames.pop()
        found = False
        for lastFrame in sortedFrames:
            if adjacent(lastFrame, nextFrame):
                sortedFrames.append(nextFrame)
                found = True
                break

        if not found:
            frames.insert(0, nextFrame)

    return sortedFrames


def adjacent(frame1, frame2):
    return hypot(frame1["x"] - frame2["x"], frame1["y"] - frame2["y"]) == 1


writeVehicleTemplates(getVehicleTemplates())



from ..write_text import write_text


def parse_vehicle_spawn(json, origin):
    for st in json.get("spawn_types", []):
        write_text(st.get("description"), origin,
                   comment="Vehicle Spawn Description")



#!/usr/bin/env python3
# pylint: disable=C0103
# pylint: disable=C0301
"""Convert a Cataclysm-DDA tileset to isometric view.
Run this script with -h for full usage information.

Examples:

    %(prog)s gfx/UltimateCataclysm
    %(prog)s gfx/BrownLikeBears

Isometric tiles are saved to a new folder like gfx/UltimateCataclysm_iso.
"""

import os
import json
import argparse
import datetime
import math

# when a 2d sprite 'stands up' on an isometric tile bae, like a tree
# how far up should it be from the bottom corner of the tile?
# offset=0 will align bottom of sprite with bottom (SW corner) of square
# offset<1 will be a fraction of the isometric tile height
# offset>=1 will be an absolute number of pixels
SPRITE_OFFSET_FROM_BOTTOM = 1.0 / 8

parser = argparse.ArgumentParser(
    description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)

parser.add_argument('tileset', help='name of the tileset directory to convert')

parser.add_argument('-f', dest='floodfill', action='store_true',
                    help='use filled background instead of transparent')

args = parser.parse_args()

# Indexed by (tile_num, initial_rotation) tuple, True if iso-converted
converted_tile_ids = dict()


def iso_ize(tile_num, initial_rotation=0, override=False):
    if override or (tile_num, initial_rotation) not in converted_tile_ids:
        print("  iso-izing " + str(tile_num))
        tile_png = "tile-{:0>6d}.png".format(tile_num)
        command = (
            'convert -background transparent ' +
            new_tileset_name + '/tiles/' + tile_png +
            ' -rotate ' + str(initial_rotation) + ' ' +
            '-rotate -45 +repage -resize 100%x50% ' +
            '-crop ' + str(nwidth) + 'x' + str(int(nwidth / 2)) +
            '+2+1 ' +  # TODO: get correct offsets
            '+repage -extent ' + str(nwidth) + 'x' + str(nheight) +
            '+0-' + str(nheight - int(nwidth / 2)) + ' ' +
            new_tileset_name + '/tiles/to_merge/' + tile_png)
        print(command)
        if os.system(command):
            raise RuntimeError("iso-ization failed for %s" % tile_png)
        else:
            converted_tile_ids[(tile_num, initial_rotation)] = True
            return True
    return False


# convert one old tile definition to one new tile definition
# recursive for additional-tiles
def tile_convert(otile, main_id, new_tile_number):
    print("tile_convert %s %s" % (main_id, new_tile_number))
    ntile = dict()
    ntile['id'] = otile['id']
    ntile['ntn'] = int(new_tile_number)

    #TODO: handle different tile transformations based on id
    #TODO: load other json files to base transforms on stats, not just id

    if 'rotates' in otile:
        ntile['rotates'] = otile['rotates']
    if 'multitile' in otile:
        ntile['multitile'] = otile['multitile']

    for g in ('fg', 'bg'):
        if g not in otile:
            continue
        if type(otile[g]) is int:
            if otile[g] == -1:
                continue
            otile[g] = list([otile[g]])

        add_tile_ids = [
            'broken',
            'open',
            'unconnected',
            'center',
            'corner',
            'edge',
            'end_piece',
            't_connection',
        ]

        # tile ids to iso-ify:
        # t_*.bg, .fg iff .bg is missing
        # t_*.additional_tiles.bg, .fg iff .bg is missing
        # lighting_*.bg, .fg iff .bg is missing
        # vp_*.bg and .fg
        # fd_*.bg and .fg
        # fd_*.additional_tiles.fg and .bg

        if len(otile[g]) == 0:
            continue
        elif len(otile[g]) == 1:
            ntile[g] = otile[g]
            # FIXME: These are factored out to (slightly) simplify the
            # conditional expression below, but it still needs cleanup
            terrain_or_lighting = isinstance(main_id, str) and (
                main_id.startswith('t_') or main_id.startswith('lighting_'))
            vehicle_part_or_field = isinstance(main_id, str) and (
                main_id.startswith('vp_') or main_id.startswith('fd_'))
            # iso-ize?
            if (
                (terrain_or_lighting and (g == 'bg' or 'bg' not in otile)) or
                (main_id[0:1] == ('f_') and g == 'bg' and 'fg' in otile) or
                vehicle_part_or_field or
                # additional_tiles:
                otile['id'] in add_tile_ids or
                ('rotates' in otile and otile['rotates'] is True) or
                otile['id'] != main_id
            ):
                # iso-ize this tile
                iso_ize(otile[g][0], override=True)
                if ('rotates' in otile or otile['id'] in add_tile_ids):
                    print("  and rotating " + str(otile[g][0]))
                    # create 3 new iso-ized tiles, as well
                    for rot in (270, 180, 90):
                        if iso_ize(otile[g][0], rot):
                            ntile[g].append(ntile['ntn'])
                            ntile['ntn'] += 1
                            print("next tile number now " + str(ntile['ntn']))
            else:
                if (ntile[g][0], 0) not in converted_tile_ids:
                    print("  offsetting " + str(ntile[g][0]))
                    converted_tile_ids[(ntile[g][0], 0)] = True
                    # offset this flat tile
                    tile_png = "tile-{:0>6d}.png".format(otile[g][0])
                    command = (
                        'convert -background transparent ' + new_tiles_dir +
                        '/' + tile_png +
                        (
                            ' -fill transparent -draw "color 0,0 floodfill"'
                            if args.floodfill else ''
                        ) +
                        ' -extent ' + str(nwidth) + 'x' + str(nheight) +
                        '-' + str(int((nwidth - owidth) / 2)) + '-' +
                        str(int((nheight - oheight) - flat_sprite_offset)) +
                        ' +repage ' + new_tiles_dir + '/to_merge/' + tile_png)
                    print(command)
                    if os.system(command):
                        print("! Failed to offset %s, continuing" % tile_png)
                        #raise RuntimeError("Failed to offset %s" % tile_png)
        else:
            ntile[g] = otile[g]
            # iso-ize each existing rotation of this tile
            for tile in ntile[g]:
                # if tile_num is a dict with "weight" and "sprite",
                # take the "sprite" number
                if type(tile) is dict and "sprite" in tile:
                    iso_ize(tile["sprite"])
                elif type(tile) is int:
                    iso_ize(tile)
                else:
                    raise RuntimeError("Unexpected sprite number: %s" % tile)

    if 'additional_tiles' in otile:
        nta = ntile['additional_tiles'] = list()
        for otile in otile['additional_tiles']:
            print(" handling additional_tile '%s'" % otile['id'])
            tile = tile_convert(otile, main_id, ntile['ntn'])
            ntile['ntn'] = tile['ntn']
            del tile['ntn']
            nta.append(tile)

    print("returning tile, next tile number now " + str(ntile['ntn']))
    return ntile


#TODO: bail out if old tileset directory doesn't exist

# FIXME: Rename to _dir, use relative path starting with 'gfx'?
old_tileset_name = os.path.abspath(args.tileset)
new_tileset_name = old_tileset_name + "_iso"
new_tiles_dir = os.path.join(new_tileset_name, 'tiles')
print("New tiles will be output to %s" % new_tiles_dir)

# Ensure empty tiles output directory
os.system('rm -rf ' + new_tiles_dir)

try:
    os.mkdir(new_tileset_name)
except OSError:
    pass

try:
    os.mkdir(os.path.join(new_tileset_name, 'tiles'))
except OSError:
    pass

os.system('rm -rf ' + new_tileset_name + '/tiles/*')

try:
    os.mkdir(os.path.join(new_tiles_dir, 'to_merge'))
except OSError:
    pass

print('reading ' + old_tileset_name + '/tile_config.json')
with open(
        old_tileset_name + '/tile_config.json',
        encoding="utf-8") as old_tile_config_json_file:
    otc = json.load(old_tile_config_json_file)

oheight = otc['tile_info'][0]['height']
owidth = otc['tile_info'][0]['width']

# iso tile width is enough to hold a 45 degree rotated original tile
nwidth = int(owidth * math.sqrt(2) / 2) * 2
# iso tile height is enough to hold a vertical offset 2d tile
if SPRITE_OFFSET_FROM_BOTTOM < 1:
    flat_sprite_offset = int(nwidth * SPRITE_OFFSET_FROM_BOTTOM / 2)
else:
    flat_sprite_offset = int(SPRITE_OFFSET_FROM_BOTTOM / 2)
print("flat sprite offset " + str(flat_sprite_offset))
nheight = flat_sprite_offset + oheight

# struct that will become the new tileset_config.json
ntc = dict()
ntc['tile_info'] = list()
ntc['tile_info'].append(dict())
ntc['tile_info'][0]['height'] = nheight
ntc['tile_info'][0]['width'] = nwidth
ntc['tile_info'][0]['iso'] = True

first_filename = ''
ntc['tiles-new'] = list()
tile_count = 0
new_tiles = list()

print('processing tiles-new')
for otn in otc['tiles-new']:
    filename = os.path.join(old_tileset_name, otn['file'])
    print(filename)
    base_filename = os.path.basename(filename)
    # need to add support for fallback tiles
    if 'fallback' in base_filename:
        continue
    if first_filename == '':
        first_filename = base_filename  # remember this for tileset.txt
    ntc['tiles-new'].append(dict())
    ntn = ntc['tiles-new'][-1]
    ntn['file'] = os.path.join(new_tileset_name, base_filename)
    ntn['tiles'] = list()

    # split tile image sheet into individual tile images
    command = (
        'convert -crop ' + str(oheight) + 'x' + str(owidth) + ' ' + filename +
        ' +repage ' + new_tiles_dir + '/tile-%06d.png')

    print(command)
    if os.system(command):
        raise RuntimeError("Failed to split %s into tile images" % filename)

    os.system(
        'cp ' + new_tiles_dir + '/tile-*.png ' + new_tiles_dir + '/to_merge')

    # path joining version for other paths
    tile_count = len([
        name for name in os.listdir(new_tiles_dir)
        if os.path.isfile(os.path.join(new_tiles_dir, name))
    ])
    new_tile_number = tile_count

    for otile in otn['tiles']:
        print(" handling '%s'" % otile['id'])
        tile = tile_convert(otile, otile['id'], new_tile_number)
        new_tile_number = tile['ntn']
        del tile['ntn']
        ntn['tiles'].append(tile)

    print('Merging tiles to single image')
    command = (
        'montage -background transparent "' + new_tileset_name +
        '/tiles/to_merge/tile-*.png" -tile 16x -geometry +0+0 ' +
        new_tileset_name + '/' + base_filename)
    print(command)
    if os.system(command):
        raise RuntimeError("Failed to merge tiles into %s" % new_tileset_name)

with open(new_tileset_name + '/tile_config.json', 'w',
        encoding="utf-8") as tile_config_json:  # noqa: E128
    json.dump(
        ntc, tile_config_json,
        sort_keys=True, indent=2, separators=(',', ': '), ensure_ascii=False)

#TODO: replace tiles.png with first filename from json
with open(
        new_tileset_name + '/tileset.txt',
        'w', encoding="utf-8") as new_tileset_txt_file:
    new_tileset_txt_file.write(
        '#Generated by make_iso.py from flat tileset ' +
        old_tileset_name + '\n' +
        '#' + datetime.datetime.now().strftime('%c') + '\n' +
        '#Name of the tileset\n' +
        'NAME: ' + new_tileset_name + '\n' +
        '#Displayed name of the tileset\n' +
        'VIEW: ' + new_tileset_name + '\n' +
        '#JSON Path - Default of gfx/tile_config.json\n' +
        'JSON: ' + new_tileset_name + '/tile_config.json\n' +
        '#Tileset Path - Default of gfx/tinytile.png\n' +
        'TILESET: ' + new_tileset_name + '/tiles.png\n'
    )



from .enchant import parse_enchant
from ..helper import get_singular_name
from .use_action import parse_use_action
from ..write_text import write_text
from .gun import parse_gun
from .gunmod import parse_gunmod
from .magazine import parse_magazine


def parse_generic(json, origin):
    name = ""
    comment = []
    if "//isbn13" in json:
        comment.append("ISBN {}".format(json["//isbn13"]))

    if "name" in json:
        name = get_singular_name(json["name"])
        write_text(json["name"], origin, comment=comment + ["Item name"],
                   plural=True, c_format=False)
    elif "id" in json:
        name = json["id"]

    if "subtypes" in json:
        subtypes_list = json["subtypes"]
        if "GUN" in subtypes_list:
            parse_gun(json, origin)
        if "GUNMOD" in subtypes_list:
            parse_gunmod(json, origin)
        if "MAGAZINE" in subtypes_list:
            parse_magazine(json, origin)

    if "description" in json:
        write_text(json["description"], origin, c_format=False,
                   comment=comment + ["Description of \"{}\"".format(name)])

    if "use_action" in json:
        parse_use_action(json["use_action"], origin, name)
    if "tick_action" in json:
        parse_use_action(json["tick_action"], origin, name)

    for cname in json.get("conditional_names", []):
        write_text(cname["name"], origin,
                   comment="Conditional name for \"{}\" when {} matches {}"
                   .format(name, cname["type"], cname["condition"]),
                   plural=True)

    if "variants" in json:
        for variant in json["variants"]:
            variant_name = get_singular_name(variant["name"])
            write_text(variant["name"], origin,
                       comment="Variant name of item \"{}\"".format(name),
                       plural=True)
            write_text(variant["description"], origin,
                       comment="Description of variant \"{1}\" of item \"{0}\""
                       .format(name, variant_name))

    if "snippet_category" in json and type(json["snippet_category"]) is list:
        # snippet_category is either a simple string (the category ident)
        # which is not translated, or an array of snippet texts.
        for entry in json["snippet_category"]:
            if type(entry) is str:
                write_text(entry, origin,
                           comment="Snippet of item \"{}\"".format(name))
            elif type(entry) is dict:
                write_text(entry["text"], origin,
                           comment="Snippet of item \"{}\"".format(name))

    if "seed_data" in json:
        write_text(json["seed_data"]["plant_name"], origin,
                   comment="Plant name of seed \"{}\"".format(name))

    if "revert_msg" in json:
        write_text(json["revert_msg"], origin,
                   comment="Dying message of tool \"{}\"".format(name))

    if "pocket_data" in json:
        for pocket in json["pocket_data"]:
            if "description" in pocket:
                write_text(pocket["description"], origin,
                           comment="Description of a pocket in item \"{}\""
                           .format(name))
            if "name" in pocket:
                write_text(pocket["name"], origin,
                           comment="Brief name of a pocket in item \"{}\""
                           .format(name))

    if "relic_data" in json and "passive_effects" in json["relic_data"]:
        for enchantment in json["relic_data"]["passive_effects"]:
            parse_enchant(enchantment, origin)



#!/usr/bin/env python3

import argparse
import json
import os

args = argparse.ArgumentParser()
args.add_argument("dir", action="store", help="specify json directory")
args_dict = vars(args.parse_args())


def portionize(jo):
    dat = dict()

    for member in ["encumbrance", "max_encumbrance",
                   "coverage", "covers"]:
        if member in jo:
            dat[member] = jo.pop(member)

    if not dat:
        return None

    if "max_encumbrance" in dat:
        if "encumbrance" not in dat:
            dat["encumbrance"] = 0
        dat["encumbrance"] = [dat["encumbrance"], dat.pop("max_encumbrance")]

    if "armor" not in jo:
        jo["armor"] = [dat]
    else:
        dat["//0"] = "Autogenned for item with non-portion and portion data"
        dat["//1"] = "Ensure that this works correctly!"
        jo["armor"].insert(0, dat)

    return jo


def gen_new(path):
    change = False
    # Having troubles with this script halting?
    # Uncomment below to find the file it's halting on
    #print(path)
    with open(path, "r", encoding="utf-8") as json_file:
        try:
            json_data = json.load(json_file)
        except json.JSONDecodeError:
            print(f"Json Decode Error at {path}")
            print("Ensure that the file is a JSON file consisting of"
                  " an array of objects!")
            return None

        for jo in json_data:
            if type(jo) is not dict:
                print(f"Incorrectly formatted JSON data at {path}")
                print("Ensure that the file is a JSON file consisting"
                      " of an array of objects!")
                break

            if ("type" not in jo or
                jo["type"] != "ARMOR" and
                jo["type"] != "TOOL_ARMOR" and
               "armor_data" not in jo):
                continue

            if "armor_data" in jo:
                new_data = portionize(jo["armor_data"])
                if new_data is not None:
                    jo["armor_data"] = new_data
                    change = True
                continue

            # I don't know python - assiging to jo is not necessary here
            # but is necessary for "armor_data" above
            new_jo = portionize(jo)
            if new_jo is not None:
                change = True

    return json_data if change else None


for root, directories, filenames in os.walk(args_dict["dir"]):
    for filename in filenames:
        if not filename.endswith(".json"):
            continue

        path = os.path.join(root, filename)
        new = gen_new(path)
        if new is not None:
            with open(path, "w", encoding="utf-8") as jf:
                json.dump(new, jf, ensure_ascii=False)
            os.system(f"./tools/format/json_formatter.cgi {path}")



from ..write_text import write_text


def parse_construction(json, origin):
    if "pre_note" in json:
        write_text(json["pre_note"], origin,
                   comment="Prerequisite of terrain constrution")



#!/usr/bin/env python3

import argparse
import json
import os

args = argparse.ArgumentParser()
args.add_argument("dir", action="store", help="specify json directory")
args_dict = vars(args.parse_args())


def to_resist_vals(jo):
    res = dict()
    if "bash_resist" in jo:
        res["bash"] = jo.pop("bash_resist")
    if "cut_resist" in jo:
        res["cut"] = jo.pop("cut_resist")
    if "acid_resist" in jo:
        res["acid"] = jo.pop("acid_resist")
    if "fire_resist" in jo:
        res["heat"] = jo.pop("fire_resist")
    if "bullet_resist" in jo:
        res["bullet"] = jo.pop("bullet_resist")
    if "elec_resist" in jo:
        res["electric"] = jo.pop("elec_resist")
    if "biologic_resist" in jo:
        res["biological"] = jo.pop("biologic_resist")
    if "cold_resist" in jo:
        res["cold"] = jo.pop("cold_resist")

    if res:
        jo["resist"] = res

    if "proportional" in jo:
        jo["proportional"] = to_resist_vals(jo["proportional"])

    if "relative" in jo:
        jo["relative"] = to_resist_vals(jo["relative"])

    return jo


def gen_new(path):
    change = False
    # Having troubles with this script halting?
    # Uncomment below to find the file it's halting on
    #print(path)
    with open(path, "r", encoding="utf-8") as json_file:
        try:
            json_data = json.load(json_file)
        except json.JSONDecodeError:
            print(f"Json Decode Error at {path}")
            print("Ensure that the file is a JSON file consisting of"
                  " an array of objects!")
            return None

        for jo in json_data:
            if type(jo) is not dict:
                print(f"Incorrectly formatted JSON data at {path}")
                print("Ensure that the file is a JSON file consisting"
                      " of an array of objects!")
                break

            if ("type" not in jo or jo["type"] != "material"):
                continue

            new_data = to_resist_vals(jo)
            if new_data is not None:
                jo = new_data
                change = True

    return json_data if change else None


for root, directories, filenames in os.walk(args_dict["dir"]):
    for filename in filenames:
        if not filename.endswith(".json"):
            continue

        path = os.path.join(root, filename)
        new = gen_new(path)
        if new is not None:
            with open(path, "w", encoding="utf-8") as jf:
                json.dump(new, jf, ensure_ascii=False)
            os.system(f"./tools/format/json_formatter.cgi {path}")



import functools
import os
import sys
from collections import defaultdict
from dataclasses import dataclass
from pathlib import Path
from typing import Generator

import click
import orjson


def find_files(
    paths: list[Path], extensions: tuple[str], include_mods=False
) -> Generator[str, None, None]:
    """Recursively search :paths: for files that have :extensions:.

    If :include_mods: is False (the default), ignore all files that are in a
    subdirectory of a directory containing a modinfo.json file. If
    :include_mods: is True, yield modinfo.json before all other files,
    if it exists.
    """
    for path in paths:
        if os.path.isdir(path):
            for dirpath, dirnames, filenames in os.walk(path):
                # Ignore all subdirectories and files inside a mod.
                if not include_mods and "modinfo.json" in filenames:
                    dirnames.clear()
                    filenames.clear()

                # Yield modinfo.json before all other files, if it exists.
                if include_mods and "modinfo.json" in filenames:
                    # More than one modinfo.json is not allowed
                    if filenames.count("modinfo.json") > 1:
                        dirnames.clear()
                        filenames.clear()
                        click.echo(
                            f"More than one modinfo.json found in {dirpath}.",
                            err=True,
                        )
                    else:
                        filenames.remove("modinfo.json")
                        yield os.path.join(dirpath, "modinfo.json")

                for filename in filenames:
                    _, ext = os.path.splitext(filename)
                    if ext in extensions:
                        yield os.path.join(dirpath, filename)
        else:
            if path.suffix in extensions:
                yield path


@functools.total_ordering
@dataclass(init=True, repr=True)
class Mutation:
    id_: str
    name: str
    categories: list[str]
    description: str
    mod: str
    copied: bool = False

    def __lt__(self, other) -> bool:
        if not isinstance(other, Mutation):
            return NotImplemented
        return self.name.lower() < other.name.lower()


def write_mutations(mutations: list[Mutation], filename: Path) -> None:
    categories = defaultdict(list)

    for mut in mutations:
        if mut.categories != "NONE":
            for cat in mut.categories:
                categories[cat].append(mut)
        else:
            categories["NONE"].append(mut)

    # Sort categories alphabetically
    categories = dict(sorted(categories.items()))

    # Sort mutations alphabetically in categories
    for mutations in categories.values():
        mutations.sort()

        # Sort each mutation's categories alphabetically
        for mut in mutations:
            mut.categories.sort()

    with open(filename, "w", encoding="utf-8") as fd:
        # Write table of contents
        fd.write("# Table of Contents\n")
        fd.write("- Categories\n")
        for category in categories.keys():
            fd.write(
                f'  - [{category}](#{category.replace(" ", "-").lower()})\n'
            )
        fd.write("\n\n")

        # Write mutations
        for category, mutations in categories.items():
            fd.write(f"## {category}\n")
            for mut in mutations:
                fd.write(f"### {mut.name}\n")
                fd.write(f"src: `{mut.mod}`\n")
                fd.write(f"id: `{mut.id_}` \n")
                fd.write(f'categories: `{"`, `".join(mut.categories)}`\n')
                fd.write(f"Description: {mut.description}\n")
            fd.write("\n\n")


def extract_mutations(file: Path, mod: str = "dda") -> list[Mutation]:
    """Extract mutations from :file:."""

    with open(file, "rb") as fd:
        jfile = orjson.loads(fd.read())

    if not isinstance(jfile, list):
        return []

    if len(jfile) == 0:
        return []

    mutations = []

    for jo in jfile:
        if not isinstance(jo, dict):
            continue

        if jo.get("type") != "mutation":
            continue

        mutation = Mutation(
            id_=jo.get("id"),
            name="",
            categories=jo.get(
                "category", jo.get("extend", {}).get("category", ["NONE"])
            ),
            description=jo.get("description"),
            mod=mod,
        )

        name = jo.get("name")
        if isinstance(name, dict):
            name = name.get("str")
        elif name is None and jo.get("copy-from"):
            name = "SHOULD_BE_COPIED_FROM"
            mutation.copied = True
        elif not isinstance(name, str):
            click.echo(
                f"Invalid mutation name: {name} in file: {file}", err=True
            )
            sys.exit()

        mutation.name = name
        mutations.append(mutation)

    return mutations


@dataclass(init=True, repr=True)
class Mod:
    id_: str
    mutations: list[Mutation]
    dependencies: list[str]


@click.command()
@click.argument(
    "paths",
    type=click.Path(exists=True, resolve_path=True, path_type=Path),
    nargs=-1,
)
@click.option(
    "-o",
    "--outfile",
    type=click.Path(resolve_path=True, path_type=Path, dir_okay=False),
    default="mutations.md",
    help="""Specify the filepath to write the documentation for DDA to.
    If mods are included, their documentation will be written to the same
    directory.""",
    show_default=True,
)
@click.option(
    "--include-mods / --exclude-mods",
    default=False,
    help="""Include (or exclude) mutations defined in mods.
    A mod is a file which is contained in a subdirectory of a
    directory including a modinfo.json file.""",
    show_default=True,
)
def cli(paths, outfile, include_mods) -> None:
    """Search PATHS for JSON files and generate documentation for mutations."""
    # The entrypoint to the CLI.

    if not include_mods:
        mutations = []

        for file in find_files(paths, (".json",), include_mods=False):
            mutations.extend(extract_mutations(file))

        write_mutations(mutations, outfile)
        click.echo(f"Mutations written to {outfile}")
    else:
        dda = Mod("dda", [], [])
        mod_stack: list[tuple[Mod, str]] = [(dda, "PLACEHOLDER")]
        mods: dict[str, Mod] = {"dda": dda}

        def move_stack(
            file, mod_stack: list[tuple[Mod, str]]
        ) -> list[tuple[Mod, str]]:
            if len(mod_stack) == 1:
                return mod_stack
            if mod_stack[-1][1] in Path(file).parents:
                return mod_stack
            else:
                return move_stack(file, mod_stack[:-1])

        for file in find_files(paths, (".json",), include_mods=True):
            if os.path.basename(file) != "modinfo.json":
                # Move mods who we have recursed out of off the stack.
                mod_stack = move_stack(file, mod_stack)
                mod_stack[-1][0].mutations.extend(
                    extract_mutations(file, mod_stack[-1][0].id_)
                )

            elif os.path.split(os.path.split(file)[0])[1] == "dda":
                # Skip DDA
                continue
            else:
                # Add a new mod to the stack.
                with open(file, "rb") as fd:
                    jfile = orjson.loads(fd.read())

                # modinfo.json supports two formats:
                # - An array containing at least one object, the first of which
                # contains info about the mod.
                # - A single top-level object.
                if isinstance(jfile, list) and len(jfile) > 0:
                    mod_info = jfile[0]
                else:
                    mod_info = jfile

                if isinstance(mod_info, dict) and all(
                    (mod_info.get("id"), mod_info.get("dependencies"))
                ):
                    new_mod = Mod(mod_info["id"], [], mod_info["dependencies"])
                    mod_stack.append((new_mod, Path(os.path.dirname(file))))
                    mods[new_mod.id_] = new_mod
                else:
                    click.echo(f"{file} is an invalid modinfo.json file.")
                    sys.exit()

        # Resolve mutations which use copy-from.
        for id_, mod in mods.items():
            if not mod.dependencies:
                continue

            for mut in (mut for mut in mod.mutations if mut.copied is True):
                for dep in mod.dependencies:
                    if not mods.get(dep):
                        click.echo(
                            f"Mod {mod.id_} is missing dependency {dep}"
                        )
                        continue

                    for m in mods[dep].mutations:
                        if m.id_ == mut.id_:
                            mut.description = m.description
                            mut.name = m.name
                            mut.mod = m.mod

                            if mut.categories != ["NONE"] and m.categories != [
                                "NONE"
                            ]:
                                mut.categories += m.categories
                            elif m.categories != ["NONE"]:
                                mut.categories = m.categories

        for _, mod in mods.items():
            try:
                if not mod.mutations:
                    continue
            except AttributeError:
                breakpoint()

            if mod.id_ == "dda":
                writepath = outfile
            else:
                writepath = outfile.parent.joinpath(mod.id_ + ".md")

            write_mutations(mod.mutations, writepath)
            click.echo(
                f"Mutations for mod [ {mod.id_} ] written to {writepath}"
            )


if __name__ == "__main__":
    cli()



from ..write_text import write_text


def parse_species(json, origin):
    id = json["id"]
    if "description" in json:
        write_text(json["description"], origin,
                   comment="Description of species \"{}\"".format(id))
    if "footsteps" in json:
        write_text(json["footsteps"], origin,
                   comment="Foot steps of species \"{}\"".format(id))



from ..helper import get_singular_name
from ..write_text import write_text


def parse_overmap_land_use_code(json, origin):
    name = get_singular_name(json.get("name", ""))
    if "name" in json:
        write_text(json["name"], origin,
                   comment="Name of an overmap land use code")
    write_text(json["detailed_definition"], origin, c_format=False,
               comment="Detailed definition of overmap land use code \"{}\""
               .format(name))



from ..write_text import write_translation_or_var


def parse_condition(conditions, origin):
    if not type(conditions) is list:
        conditions = [conditions]
    for cond in conditions:
        if type(cond) is dict:
            # if *_query is an object, the message is taken from elsewhere
            if "u_query" in cond and type(cond["u_query"]) is str:
                write_translation_or_var(cond["u_query"], origin,
                                         comment="Query message shown in a "
                                         "popup")
            if "npc_query" in cond and type(cond["npc_query"]) is str:
                write_translation_or_var(cond["npc_query"], origin,
                                         comment="Query message shown in a "
                                         "popup")
            if "and" in cond:
                parse_condition(cond["and"], origin)
            if "or" in cond:
                parse_condition(cond["or"], origin)
            if "not" in cond:
                parse_condition(cond["not"], origin)



#!/usr/bin/env python3

# Usage 'convert_armor.py dir'
# NOTE: for mods, ensure 'dir' includes as a subdirectory all
#       JSON data which an modded armor item may include (e.g. data/json)

import argparse
import json
import os


args = argparse.ArgumentParser()
args.add_argument("dir", action="store", help="specify json directory")
args_dict = vars(args.parse_args())

# All the file paths of valid JSON files
# we read all the files twice, may as well save them
all_filepaths = set()
# Definitions and paths:
items_at_path = dict()
# All of the armor definitions - for copy-from
all_armor_defs = dict()
# The keys we are moving to armor portion data
transferred_keys = [
    "material",
    "material_thickness",
    "environmental_protection",
    "environmental_protection_with_filter"
]
# Item types we might be copying from
accepted_types = [
    "AMMO",
    "GUN",
    "ARMOR",
    "PET_ARMOR",
    "TOOL",
    "TOOLMOD",
    "TOOL_ARMOR",
    "BOOK",
    "COMESTIBLE",
    "EGINE",
    "WHEEL",
    "GUNMOD",
    "MAGAZINE",
    "BATTERY",
    "GENERIC",
    "BIONIC_ITEM"
]


def id_or_abstract(jo):
    if "id" in jo:
        return jo["id"]
    return jo["abstract"]


# Extract all the JSON defs of armor items to all_armor_defs for copy-from
# And store the paths of all correctly formatted files for later use
def extract_armor_add_path(path):
    with open(path, "r", encoding="utf-8") as json_file:
        try:
            json_data = json.load(json_file)
        except json.JSONDecodeError:
            print(f"Json Decode Error at {path}")
            print("Ensure that the file is a JSON file consisting of"
                  " an array of objects!\n")
            return None

        for jo in json_data:
            if type(jo) is not dict:
                print(f"Incorrectly formatted JSON data at {path}")
                print("Ensure that the file is a JSON file consisting"
                      " of an array of objects!\n")
                break

            all_filepaths.add(path)

            if ("type" not in jo or
               jo["type"] not in accepted_types):
                continue

            def_id = id_or_abstract(jo)

            if def_id not in items_at_path:
                items_at_path[def_id] = set()
            items_at_path[def_id].add(path)

            # Remove comments for better duplicate checking
            '''
            color_discard = "\x1b[31m"
            color_used = "\x1b[32m"
            color_end = "\x1b[0m"
            '''

            if (def_id in all_armor_defs and not
               ("copy-from" in all_armor_defs[def_id] and
                   all_armor_defs[def_id]["copy-from"] == def_id)):
                print(f"WARNING: {def_id} defined twice, at {path},"
                      " second definition discarded")
                # Here too
                '''
                print("Defined at:")
                for path in items_at_path[def_id]:
                    print(f"! '{def_id}': {color_discard}{path}{color_end}")
                '''
                print("This can occur due to mods overriding items, "
                      "ensure that the assigned values are accurate!\n")
                return
            elif def_id in all_armor_defs:
                print(f"WARNING: {def_id} defined twice, "
                      "using definition from {path}")
                # And here
                '''
                print("Defined at:")
                for path in items_at_path[def_id]:
                    print(f"! '{def_id}': {color_used}{path}{color_end}")
                '''
                print("This can occur due to mods overriding items, "
                      "ensure that the assigned values are accurate!\n")

            all_armor_defs[def_id] = jo


def assign_if_absent(dat, key, value):
    if key not in dat:
        dat[key] = value


# Store any of the keys we're transferring into dict, not overwriting existing
def read_armor_data(jo, dat):
    for key in transferred_keys:
        if key in jo:
            assign_if_absent(dat, key, jo[key])


# while there's still armor data to get
# grab it from each definition in the copy-from chain
def get_armor_data_rec(jo, dat):
    # When we've got all of our data filled, we can stop recursing
    if all(key in dat for key in transferred_keys):
        return

    read_armor_data(jo, dat)

    if "copy-from" in jo:
        get_armor_data_rec(all_armor_defs[jo["copy-from"]], dat)


# Get the armor data for this item that needs to be transferred
# Returns empty dict if this armor specifies no novel armor data
# (And doesn't already have other armor data)
def get_armor_data(jo):
    dat = dict()
    # This will be done again in the recursive function
    # But we do it here for an early return on no data
    read_armor_data(jo, dat)
    if ("armor" not in jo and
       (len(dat) == 0 or (len(dat) == 1 and "material" in dat))):
        return dict()

    get_armor_data_rec(jo, dat)
    return dat


# If this item has data that is actually distinct from the parent item
# So we can do nothing in cases where copy-from handles it all
def actually_distinct(jo):
    if "copy-from" not in jo:
        return True

    dat = dict()

    read_armor_data(jo, dat)

    copied = get_armor_data(all_armor_defs[jo["copy-from"]])

    for key in dat:
        if key not in copied:
            return True
        if copied[key] != dat[key]:
            return True

    return False


# Merge the 'armor' section of the parent item and the new data
def merge_with_armor(jo, dat):
    orig = all_armor_defs[jo["copy-from"]]
    while "armor" not in orig:
        if "copy-from" not in orig:
            return [dat]
        orig = all_armor_defs[orig["copy-from"]]

    out = orig["armor"]
    for portion in out:
        for key in dat:
            portion[key] = dat[key]

    return out


def handle_armor_data(jo):
    armor = jo["armor_data"]
    dat = dict()
    for key in transferred_keys:
        if key not in armor:
            continue
        dat[key] = armor[key]
        del armor[key]

    curjo = jo
    while "material" not in curjo:
        out = id_or_abstract(curjo)
        if "copy-from" not in curjo:
            print(f"{out}: malformed armor def - materials are required"
                  " for armor items")
            return
        if curjo["copy-from"] not in all_armor_defs:
            print(f"{out}: does not copy-from an armor item!"
                  "material will need manual addition")
            return
        if curjo["copy-from"] == out:
            print("AAAAAAA")
            print(f"{out}")
        curjo = all_armor_defs[curjo["copy-from"]]

    dat["material"] = curjo["material"]

    if "armor" in armor:
        for portion in armor["armor"]:
            portion.update(dat)

    return jo


def gen_new(path):
    change = False
    with open(path, "r", encoding="utf-8") as json_file:
        try:
            json_data = json.load(json_file)
        except json.JSONDecodeError:
            print(f"Json Decode Error at {path}")
            print("Ensure that the file is a JSON file consisting of"
                  " an array of objects!")
            return None

        for jo in json_data:
            if ("type" not in jo or
               jo["type"] != "ARMOR" and
               jo["type"] != "TOOL_ARMOR" and
               "armor_data" not in jo):
                continue

            if "armor_data" in jo:
                change = True
                handle_armor_data(jo)
                continue

            dat = get_armor_data(jo)
            if len(dat) == 0:
                continue

            change = True

            if "armor" not in jo:
                if actually_distinct(jo):
                    if "copy-from" in jo:
                        jo["armor"] = merge_with_armor(jo, dat)
                    else:
                        jo["armor"] = [dat]
            else:
                for portion in jo["armor"]:
                    portion.update(dat)

            for key in transferred_keys:
                if key != "material" and key in jo:
                    del jo[key]

        return json_data if change else None


print("extracting data...")
for root, directories, filenames in os.walk(args_dict["dir"]):
    for filename in filenames:
        if not filename.endswith(".json"):
            continue

        path = os.path.join(root, filename)
        extract_armor_add_path(path)

print("Done!")
print("Modifying...")
for path in all_filepaths:
    new = gen_new(path)
    if new is not None:
        with open(path, "w", encoding="utf-8") as jf:
            json.dump(new, jf, ensure_ascii=False)
        os.system(f"./tools/format/json_formatter.cgi {path}")



import re


def get_singular_name(name):
    if type(name) is dict:
        if "str_sp" in name:
            return name["str_sp"]
        elif "str" in name:
            return name["str"]
        else:
            raise Exception("Cannot find singular name in {}".format(name))
    elif type(name) is str:
        return name
    else:
        raise Exception("Cannot find singular name in {}".format(name))


tag_pattern = re.compile(r'^<[a-z0-9_]*>$')


def is_tag(text):
    return tag_pattern.match(text)



from ..helper import get_singular_name
from ..write_text import write_text
from .effect import parse_effect
from .mapgen import parse_mapgen_object


def parse_mission_definition(json, origin):
    name = get_singular_name(json["name"])
    write_text(json["name"], origin, comment="Mission name")

    if "description" in json:
        write_text(json["description"], origin,
                   comment="Description of mission \"{}\"".format(name))

    if "dialogue" in json:
        for key in ["describe", "offer", "accepted", "rejected", "advice",
                    "inquire", "success", "success_lie", "failure"]:
            if key in json["dialogue"]:
                write_text(json["dialogue"][key], origin,
                           comment="Dialogue line in mission \"{}\"".
                           format(name))

    for key in ["start", "end", "fail"]:
        if key in json:
            if "effect" in json[key]:
                parse_effect(json[key]["effect"], origin)
            if "update_mapgen" in json[key]:
                parse_mapgen_object(json[key]["update_mapgen"], origin,
                                    om="update on {} of mission \"{}\""
                                    .format(key, name))



from ..write_text import write_text


def parse_tool_quality(json, origin):
    if "name" in json:
        write_text(json["name"], origin,
                   comment="Name of tool quality \"{}\"".format(json["id"]))



#!/usr/bin/env python3
"""Taking a filtered list of JSON data, look for all items that
match the where clause and output to stdout. All items that don't
match are output to stderr.

Example usages:

    # Anything of type dream and strength 2 goes to stdout
    # everything else goes to stderr
    %(prog)s --fnmatch=dreams.json type=dream strength=2 \\
        1>matched.json 2>not_matched.json

"""

import sys
import argparse
from util import import_data, matches_all_wheres, CDDAJSONWriter, WhereAction

parser = argparse.ArgumentParser(
    description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)
parser.add_argument(
    "--fnmatch",
    default="*.json",
    help="override with glob expression to select a smaller fileset.")
parser.add_argument(
    "where",
    action=WhereAction, nargs='+', type=str,
    help="where exclusions of the form 'where_key=where_val', no quotes.")


if __name__ == "__main__":
    args = parser.parse_args()

    json_data, load_errors = import_data(json_fmatch=args.fnmatch)
    if load_errors:
        # If we start getting unexpected JSON or other things, might need to
        # revisit quitting on load_errors
        print("Error loading JSON data.")
        for e in load_errors:
            print(e)
        sys.exit(1)
    elif not json_data:
        print("No data loaded.")
        sys.exit(1)

    matched = []
    not_matched = []
    for item in json_data:
        if matches_all_wheres(item, args.where):
            matched.append(item)
        else:
            not_matched.append(item)

    # matched first
    print("[")
    for i, p in enumerate(matched):
        eol = ",\n" if i < len(matched) - 1 else "\n"
        print(CDDAJSONWriter(p, 1).dumps(), end=eol)
    print("]")

    # not_matched second
    print("[", file=sys.stderr)
    for i, p in enumerate(not_matched):
        eol = ",\n" if i < len(not_matched) - 1 else "\n"
        print(CDDAJSONWriter(p, 1).dumps(), end=eol, file=sys.stderr)
    print("]", file=sys.stderr)



#!/usr/bin/env python3

import argparse
import json
import os

args = argparse.ArgumentParser()
args.add_argument("dir", action="store", help="specify json directory")
args_dict = vars(args.parse_args())


def gen_new(path):
    change = False
    with open(path, "r", encoding="utf-8") as json_file:
        json_data = json.load(json_file)

        for jo in json_data:
            if type(jo) is not dict or "type" not in jo:
                continue

            if "proficiencies" in jo:
                for prof in jo["proficiencies"]:
                    if "fail_multiplier" in prof:
                        prof["skill_penalty"] = prof["fail_multiplier"] - 1
                        del prof["fail_multiplier"]
                        change = True

            elif (jo["type"] == "proficiency" and
                  "default_fail_multiplier" in jo):
                jo["default_skill_penalty"] = jo["default_fail_multiplier"] - 1
                del jo["default_fail_multiplier"]
                change = True

    return json_data if change else None


for root, directories, filenames in os.walk(args_dict["dir"]):
    for filename in filenames:
        path = os.path.join(root, filename)
        if path.endswith(".json"):
            new = gen_new(path)
            if new is not None:
                with open(path, "w", encoding="utf-8") as jf:
                    json.dump(new, jf, ensure_ascii=False)
                os.system(f"./tools/format/json_formatter.cgi {path}")



from ..write_text import write_text


def parse_speech(json, origin):
    speaker = ""
    if "speaker" in json:
        if type(json["speaker"]) is list:
            speaker = ", ".join(json.get("speaker", []))
        elif type(json["speaker"]) is str:
            speaker = json["speaker"]
    if "sound" in json:
        write_text(json["sound"], origin, c_format=False,
                   comment="Speech from speaker {}".format(speaker))



#!/usr/bin/env python3
"""
List all IDs that are present in a tileset

./list_tileset_ids.py ../../gfx/UltimateCataclysm/
"""

import argparse
import json
import os


TILE_CONFIG_NAME = 'tile_config.json'
SPRITES_LIST_KEY = 'tiles-new'
TILES_LIST_KEY = 'tiles'

parser = argparse.ArgumentParser(
    description="List all IDs that are present in a tileset")
parser.add_argument("tileset_dir", action="store",
                    help="local name of the tileset directory")
args = parser.parse_args()

tileset_dirname = args.tileset_dir

# loading data
with open(
        os.path.join(tileset_dirname, TILE_CONFIG_NAME),
        encoding="utf-8") as fh:
    sprites = json.load(fh)[SPRITES_LIST_KEY]

for sprite_data in sprites:
    for tile in sprite_data[TILES_LIST_KEY]:
        tile_id_container = tile['id']
        if isinstance(tile_id_container, list):
            for tile_id in tile_id_container:
                print(tile_id)
        else:
            print(tile_id_container)



from ..helper import get_singular_name
from ..write_text import write_text


def parse_mutation_category(json, origin):
    name = get_singular_name(json["name"])
    write_text(json["name"], origin, comment="Mutation class name")

    for key in ["mutagen_message",
                "iv_message",
                "iv_sleep_message",
                "iv_sound_message",
                "junkie_message"
                ]:
        if key in json:
            write_text(json[key], origin,
                       comment="\"{}\" of mutation class \"{}\""
                       .format(key, name))

    if "memorial_message" in json:
        write_text(json["memorial_message"], origin,
                   context="memorial_male",
                   comment="Memorial message of mutation class "
                   "\"{}\" for male".format(name))
        write_text(json["memorial_message"], origin,
                   context="memorial_female",
                   comment="Memorial message of mutation class "
                   "\"{}\" for female".format(name))



from ..helper import get_singular_name
from ..write_text import write_text


def parse_martial_art(json, origin):
    name = get_singular_name(json["name"])
    write_text(name, origin, comment="Name of martial art")

    if "description" in json:
        write_text(json["description"], origin, c_format=False,
                   comment="Description of martial art \"{}\"".format(name))

    if "initiate" in json:
        messages = json["initiate"]
        if type(messages) is str:
            messages = [messages]
        for msg in messages:
            write_text(msg, origin,
                       comment="Initiate message of martial art \"{}\"".
                       format(name))

    onhit_buffs = json.get("onhit_buffs", list())
    static_buffs = json.get("static_buffs", list())
    onmove_buffs = json.get("onmove_buffs", list())
    ondodge_buffs = json.get("ondodge_buffs", list())
    onattack_buffs = json.get("onattack_buffs", list())
    onpause_buffs = json.get("onpause_buffs", list())
    onblock_buffs = json.get("onblock_buffs", list())
    ongethit_buffs = json.get("ongethit_buffs", list())
    onmiss_buffs = json.get("onmiss_buffs", list())
    oncrit_buffs = json.get("oncrit_buffs", list())
    onkill_buffs = json.get("onkill_buffs", list())

    buffs = (onhit_buffs + static_buffs + onmove_buffs + ondodge_buffs +
             onattack_buffs + onpause_buffs + onblock_buffs + ongethit_buffs +
             onmiss_buffs + oncrit_buffs + onkill_buffs)

    for buff in buffs:
        buff_name = get_singular_name(buff["name"])
        write_text(buff_name, origin,
                   comment="Buff name of martial art \"{}\"".format(name))
        write_text(buff["description"], origin, c_format=False,
                   comment="Description of buff \"{0}\" in martial art "
                           "\"{1}\"".format(name, buff_name))
